'use strict';

exports.__esModule = true;
exports.default = getRealPath;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _mapToRelative = require('./mapToRelative');

var _mapToRelative2 = _interopRequireDefault(_mapToRelative);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function findPathInRoots(sourcePath, _ref) {
  var extensions = _ref.extensions,
      root = _ref.root;

  // Search the source path inside every custom root directory
  var resolvedSourceFile = void 0;

  root.some(function (basedir) {
    try {
      // Check if the file exists (will throw if not)
      resolvedSourceFile = _resolve2.default.sync(`./${sourcePath}`, {
        basedir,
        extensions
      });
      return true;
    } catch (e) {
      return false;
    }
  });

  return resolvedSourceFile;
}

function getRealPathFromRootConfig(sourcePath, currentFile, opts) {
  var absFileInRoot = findPathInRoots(sourcePath, opts);

  if (!absFileInRoot) {
    return null;
  }

  var realSourceFileExtension = _path2.default.extname(absFileInRoot);
  var sourceFileExtension = _path2.default.extname(sourcePath);

  // Map the source and keep its extension if the import/require had one
  var ext = realSourceFileExtension === sourceFileExtension ? realSourceFileExtension : '';
  return (0, _utils.toLocalPath)((0, _utils.toPosixPath)((0, _utils.replaceExtension)((0, _mapToRelative2.default)(opts.cwd, currentFile, absFileInRoot), ext)));
}

function getRealPathFromAliasConfig(sourcePath, currentFile, _ref2) {
  var alias = _ref2.alias,
      cwd = _ref2.cwd;

  var moduleSplit = sourcePath.split('/');
  var aliasPath = void 0;

  while (moduleSplit.length) {
    var m = moduleSplit.join('/');
    if ({}.hasOwnProperty.call(alias, m)) {
      aliasPath = alias[m];
      break;
    }
    moduleSplit.pop();
  }

  // no alias mapping found
  if (!aliasPath) {
    return null;
  }

  // remove legacy "npm:" prefix for npm packages
  aliasPath = aliasPath.replace(/^(npm:)/, '');
  var newPath = sourcePath.replace(moduleSplit.join('/'), aliasPath);

  // alias to npm module don't need relative mapping
  if (aliasPath[0] !== '.') {
    return newPath;
  }

  return (0, _utils.toLocalPath)((0, _utils.toPosixPath)((0, _mapToRelative2.default)(cwd, currentFile, newPath)));
}

function getRealPathFromRegExpConfig(sourcePath, currentFile, _ref3) {
  var regExps = _ref3.regExps;

  var aliasedSourceFile = void 0;

  regExps.find(function (_ref4) {
    var regExp = _ref4[0],
        substitute = _ref4[1];

    var execResult = regExp.exec(sourcePath);

    if (execResult === null) {
      return false;
    }

    aliasedSourceFile = substitute(execResult);
    return true;
  });

  return aliasedSourceFile;
}

var resolvers = [getRealPathFromRootConfig, getRealPathFromAliasConfig, getRealPathFromRegExpConfig];

function getRealPath(sourcePath, _ref5) {
  var file = _ref5.file,
      opts = _ref5.opts;

  if (sourcePath[0] === '.') {
    return sourcePath;
  }

  // File param is a relative path from the environment current working directory
  // (not from cwd param)
  var currentFile = _path2.default.resolve(file.opts.filename);
  var resolvedPath = null;

  resolvers.some(function (resolver) {
    resolvedPath = resolver(sourcePath, currentFile, opts);
    return resolvedPath !== null;
  });

  return resolvedPath;
}