'use strict';

exports.__esModule = true;
exports.toPosixPath = toPosixPath;
exports.toLocalPath = toLocalPath;
exports.replaceExtension = replaceExtension;
exports.matchesPattern = matchesPattern;
exports.mapPathString = mapPathString;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _getRealPath = require('./getRealPath');

var _getRealPath2 = _interopRequireDefault(_getRealPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toPosixPath(modulePath) {
  return modulePath.replace(/\\/g, '/');
}

function toLocalPath(modulePath) {
  return modulePath.replace(/\/index$/, '') // remove trailing /index
  .replace(/^(?!\.)/, './'); // insert `./` to make it a local path
}

function replaceExtension(modulePath, ext) {
  var filename = _path2.default.basename(modulePath, _path2.default.extname(modulePath)) + ext;
  return _path2.default.join(_path2.default.dirname(modulePath), filename);
}

function matchesPattern(types, calleePath, pattern) {
  var node = calleePath.node;


  if (types.isMemberExpression(node)) {
    return calleePath.matchesPattern(pattern);
  }

  if (!types.isIdentifier(node) || pattern.includes('.')) {
    return false;
  }

  var name = pattern.split('.')[0];

  return node.name === name;
}

function mapPathString(nodePath, state) {
  if (!state.types.isStringLiteral(nodePath)) {
    return;
  }

  var modulePath = (0, _getRealPath2.default)(nodePath.node.value, state);
  if (modulePath) {
    nodePath.replaceWith(state.types.stringLiteral(modulePath));
  }
}